from .. import ContentsClassifier


class Derivative(ContentsClassifier):
    """An image type that is derived from other images"""


class Mask(Derivative):
    """A binary image that is multiplied with a real-valued image to select sections
    of the image for analysis"""


class FibreOrientationDistribution(Derivative):
    pass


FOD = FibreOrientationDistribution



from fileformats.generic import File


class PetRawData(File):

    binary = True
    iana_mime = None


class PetListMode(PetRawData):
    "raw projection data"
    iana_mime = None


class PetSinogram(PetRawData):
    "histogrammed projection data in a reconstruction-friendly format"
    iana_mime = None


class PetCountRate(PetRawData):
    "number of prompt/random/single events per unit time"
    iana_mime = None


class PetNormalisation(PetRawData):
    "normalisation scan or the current cross calibration factor"
    iana_mime = None



from pathlib import Path
import typing as ty
import nibabel
import numpy as np
from fileformats.core import FileSet, SampleFileGenerator
from fileformats.medimage import MedicalImage, Nifti, NiftiGz, Nifti1, NiftiGzX, NiftiX
import medimages4tests.dummy.nifti


@FileSet.read_metadata.register
def nifti_read_metadata(nifti: Nifti) -> ty.Mapping[str, ty.Any]:
    return dict(nibabel.load(nifti.fspath).header)


@MedicalImage.read_array.register
def nifti_data_array(nifti: Nifti) -> np.ndarray:  # noqa
    return nibabel.load(nifti.fspath).get_data()


@MedicalImage.vox_sizes.register
def nifti_vox_sizes(nifti: Nifti) -> ty.Tuple[float, float, float]:
    # FIXME: This won't work for 4-D files
    return tuple(float(d) for d in nifti.metadata["pixdim"][1:4])


@MedicalImage.dims.register
def nifti_dims(nifti: Nifti) -> ty.Tuple[int, int, int]:
    # FIXME: This won't work for 4-D files
    return tuple(int(d) for d in nifti.metadata["dim"][1:4])


@FileSet.generate_sample_data.register
def nifti_generate_sample_data(
    nifti: Nifti1,
    generator: SampleFileGenerator,
) -> ty.Iterable[Path]:
    return medimages4tests.dummy.nifti.get_image(
        out_file=generator.generate_fspath(file_type=Nifti1)
    )


@FileSet.generate_sample_data.register
def nifti_gz_generate_sample_data(
    nifti: NiftiGz,
    generator: SampleFileGenerator,
) -> ty.Iterable[Path]:
    return medimages4tests.dummy.nifti.get_image(
        out_file=generator.generate_fspath(file_type=NiftiGz),
        compressed=True,
    )


@FileSet.generate_sample_data.register
def nifti_gz_x_generate_sample_data(
    nifti: NiftiGzX,
    generator: SampleFileGenerator,
) -> ty.Iterable[Path]:
    return medimages4tests.mri.neuro.t1w.get_image(out_dir=generator.dest_dir)


@FileSet.generate_sample_data.register
def nifti_x_generate_sample_data(
    nifti: NiftiX,
    generator: SampleFileGenerator,
) -> ty.Iterable[Path]:
    nifti_gz_x = NiftiGzX(medimages4tests.mri.neuro.t1w.get_image(out_dir=generator.dest_dir))
    return NiftiX.convert(nifti_gz_x)



import os
import logging
import tempfile
from pathlib import Path
import pytest
try:
    from pydra import set_input_validator
except ImportError:
    pass
else:
    set_input_validator(True)
from fileformats.medimage.dicom import DicomDir

try:
    import pydra.tasks.mrtrix3.v3_0  # noqa
except ImportError:
    OLD_MRTRIX_VERSION = True
else:
    OLD_MRTRIX_VERSION = False

# Set DEBUG logging for unittests

log_level = logging.WARNING

logger = logging.getLogger("arcana")
logger.setLevel(log_level)

sch = logging.StreamHandler()
sch.setLevel(log_level)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
sch.setFormatter(formatter)
logger.addHandler(sch)


@pytest.fixture(scope="session")
def dummy_t1w_dicom():
    import medimages4tests.dummy.dicom.mri.t1w.siemens.skyra.syngo_d13c as module

    return DicomDir(module.get_image())


@pytest.fixture(scope="session")
def dummy_magfmap_dicom():
    import medimages4tests.dummy.dicom.mri.fmap.siemens.skyra.syngo_d13c as module

    return DicomDir(module.get_image())


@pytest.fixture(scope="session")
def dummy_dwi_dicom():
    import medimages4tests.dummy.dicom.mri.dwi.siemens.skyra.syngo_d13c as module

    return DicomDir(module.get_image())


@pytest.fixture(scope="session")
def dummy_mixedfmap_dicom():
    import medimages4tests.dummy.dicom.mri.fmap.ge.discovery_mr888.dv26_0_r05_2008a as module

    return DicomDir(module.get_image())


@pytest.fixture(scope="session")
def dummy_dwi_dicom():
    import medimages4tests.dummy.dicom.mri.dwi.siemens.skyra.syngo_d13c as module

    return DicomDir(module.get_image())

# For debugging in IDE's don't catch raised exceptions and let the IDE
# break at it
if os.getenv("_PYTEST_RAISE", "0") != "0":

    @pytest.hookimpl(tryfirst=True)
    def pytest_exception_interact(call):
        raise call.excinfo.value

    @pytest.hookimpl(tryfirst=True)
    def pytest_internalerror(excinfo):
        raise excinfo.value


@pytest.fixture
def work_dir():
    work_dir = tempfile.mkdtemp()
    return Path(work_dir)



import typing as ty
import logging
from fileformats.core import hook, FileSet
from fileformats.core.mixin import WithClassifiers
from .contents import ContentsClassifier
from .contents.imaging.modality import ImagingModality
from .contents.imaging.derivatives import Derivative
from .contents.anatomical_entity.material_anatomical_entity import AnatomicalEntity

logger = logging.getLogger("fileformats")


# =====================================================================
# Custom loader functions for different image types
# =====================================================================


class MedicalImage(WithClassifiers, FileSet):

    iana_mime: ty.Optional[str] = None
    INCLUDE_HDR_KEYS = None
    IGNORE_HDR_KEYS = None
    binary = True
    allowed_classifiers = (ContentsClassifier,)
    exclusive_classifiers = (ImagingModality, AnatomicalEntity, Derivative)

    @hook.extra
    def read_array(self) -> "numpy.ndarray":  # noqa
        """
        Returns the binary data of the image in a numpy array
        """
        raise NotImplementedError

    @hook.extra
    def vox_sizes(self) -> ty.Tuple[float, float, float]:
        """The length of the voxels along each dimension"""
        raise NotImplementedError

    @hook.extra
    def dims(self) -> ty.Tuple[int, int, int]:
        """The dimensions of the image"""
        raise NotImplementedError



import typing as ty
from fileformats.core import FileSet
from fileformats.application import Xml


class SurfaceMesh(FileSet):

    iana_mime: ty.Optional[str] = None


class Gifti(SurfaceMesh, Xml):

    ext = ".gii"



import typing as ty
from fileformats.core.mixin import WithMagicNumber
from fileformats.image import (
    RasterImage,
    VectorImage,
    Bitmap,
    Jpeg,
    Tiff,
)
from fileformats.application import Dicom, Gzip
from .nifti import Nifti1, NiftiGz


class GDCM(Dicom):
    pass


class GIPL(RasterImage):
    ext = ".gipl"


class VTK(VectorImage):
    ext = ".vtk"


class PGM(RasterImage):
    ext = ".pgm"


class MetaImage(RasterImage):
    ext = ".mhd"


class Nrrd(WithMagicNumber, RasterImage):

    ext = ".nrrd"
    alternate_exts = (".nhdr",)
    magic_number = b"NRRD"


class NrrdGz(Gzip[Nrrd]):

    ext = ".nrrd.gz"
    alternate_exts = (".nhdr.gz",)


ItkImage = ty.Union[
    Nifti1, NiftiGz, Dicom, Bitmap, Tiff, Jpeg, GIPL, MetaImage, Nrrd, NrrdGz, PGM
]

ItkAll = ty.Union[
    Nifti1, NiftiGz, Dicom, Bitmap, Tiff, Jpeg, GIPL, MetaImage, Nrrd, NrrdGz, PGM, VTK
]
